﻿--[[ Postal Express: Mouse Shortcuts for Mail ]]--
--shift-click to take item/money from mail
--ctrl-click to return mail
--alt-click to add item to SendMailFrame

assert( Postal, "Postal not found!")

------------------------------
--      Are you local?      --
------------------------------

local lib = AceLibrary("PostalLib-1.0")
local L = AceLibrary("AceLocale-2.2"):new("Postal")

----------------------------------
--      Module Declaration      --
----------------------------------

Postal_Express = Postal:NewModule("Express")
Postal_Express.revision = tonumber(string.sub("$Revision: 55168 $", 12, -3))

function Postal_Express:MAIL_SHOW()
	self:HookScript(GameTooltip, "OnTooltipSetItem")
	self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
	self:RegisterEvent("MAIL_CLOSED", "UnhookStuff")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "UnhookStuff")
end

function Postal_Express:UnhookStuff()
	if self:IsHooked(GameTooltip, "OnTooltipSetItem") then
		self:Unhook(GameTooltip, "OnTooltipSetItem")
		self:Unhook("ContainerFrameItemButton_OnModifiedClick")
	end
	self:UnregisterEvent("MAIL_CLOSED")
	self:UnregisterEvent("PLAYER_LEAVING_WORLD")
end
	
function Postal_Express:OnEnable()
	self:Hook("InboxFrame_OnClick", true)
	self:Hook("InboxFrameItem_OnEnter", true)

	self:RegisterEvent("MAIL_SHOW")
	self:RegisterEvent("MAIL_CLOSED", "UnhookStuff")	-- Required for AddonLoader
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "UnhookStuff")
end

-- Ctrl-Click and Shift-Clicks Shortcuts for Mail, based on ExpressMail by Tekkub
local elapsedTime = 0
function Postal_Express:InboxFrameItem_OnEnter()
	-- Apparently, Blizzard default UI's code has an OnUpdate that runs this function as fast as it can
	-- So throttle it to every TOOLTIP_UPDATE_TIME seconds, which is 0.9, so that Blizzard's code doesn't
	-- call GameTooltip:SetInboxitem(index) every frame and calling the 1000 addons out there that
	-- hook it, like RatingBuster and EnhTooltip. Really Blizzard...
	if type(arg1) == "number" then
		-- If arg1 is a number, then this was called from an OnUpdate in the default UI's MailItemTemplate
		-- OnUpdate function rather than from the OnEnter function
		elapsedTime = elapsedTime + arg1
		if elapsedTime < TOOLTIP_UPDATE_TIME then return end
		elapsedTime = 0
	end
	self.hooks["InboxFrameItem_OnEnter"]()

	local tooltip = GameTooltip
	local this = this
	
	local money, COD, _, hasItem, _, wasReturned, _, canReply = select(5, GetInboxHeaderInfo(this.index))
	if hasItem and hasItem > 1 then
		for i = 1, ATTACHMENTS_MAX_RECEIVE do
			local name, itemTexture, count, quality, canUse = GetInboxItem(this.index, i);
			if name then
				local itemLink = GetInboxItemLink(this.index, i);
				if count > 1 then
					tooltip:AddLine(("  %sx%d"):format(itemLink, count))
				else
					tooltip:AddLine("  "..itemLink)
				end
			end
		end
	end
	if money > 0 or COD > 0 or hasItem then
		tooltip:AddLine(L["|cffeda55fShift-Click|r to take the contents."])
	end
	if not wasReturned and canReply then
		tooltip:AddLine(L["|cffeda55fCtrl-Click|r to return it to sender."])
	end
	tooltip:Show()
end

function Postal_Express:InboxFrame_OnClick(index)
	if IsShiftKeyDown() then
		lib:PurgeMail(index, 1)
	elseif IsControlKeyDown() then
		lib:ReturnMail(index)
	else
		self.hooks["InboxFrame_OnClick"](index)
	end
end

function Postal_Express:OnTooltipSetItem(tooltip, ...)
	local recipient = SendMailNameEditBox:GetText()
	if type(recipient) == "string" and recipient:len() > 0 and SendMailFrame:IsVisible() and not CursorHasItem() then
		tooltip:AddLine(string.format(L["|cffeda55fAlt-Click|r to send this item to %s."], recipient))
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
end

function Postal_Express:ContainerFrameItemButton_OnModifiedClick(...)
	if select(1, ...) == "LeftButton" and IsAltKeyDown() and SendMailFrame:IsVisible() and not CursorHasItem() then
		local bag, slot = this:GetParent():GetID(), this:GetID()
		local texture, count = GetContainerItemInfo(bag, slot)
		PickupContainerItem(bag, slot)
		ClickSendMailItemButton()
		local smf_texture, smf_count = select(2, GetSendMailItem())
		if SendMailNameEditBox:GetText() and texture == smf_texture and count == smf_count then
			SendMailFrame_SendMail()
		end
	else
		self.hooks["ContainerFrameItemButton_OnModifiedClick"](...)
	end
end

